#ifndef oxygenpropertynames_h
#define oxygenpropertynames_h

/*
 * oxygenpropertynames.h
 * hard-coded property names to disable some of oxygen features
 * -------------------
 *
 * SPDX-FileCopyrightText: 2012 Hugo Pereira Da Costa <hugo.pereira@free.fr>
 * SPDX-FileCopyrightText: 2003 - 2010 Craig Drummond <craig.p.drummond@gmail.com>
 * SPDX-License-Identifier: LGPL-2.0-or-later
 *
 * Largely inspired from Qtcurve style
 */

namespace Oxygen
{
struct PropertyNames {
    static const char noAnimations[];
    static const char noWindowGrab[];
    static const char netWMForceShadow[];
    static const char netWMSkipShadow[];
    static const char toolButtonAlignment[];
};
}

#endif
